@@ -19,7 +19,7 @@ from .. import mesonlib
 from .. import compilers
 import json
 import subprocess
-from ..mesonlib import MesonException
+from ..mesonlib import MesonException, get_compiler_for_source, classify_unity_sources
 
 class InstallData():
     def __init__(self, source_dir, build_dir, prefix):
@@ -78,31 +78,16 @@ class Backend():
             priv_dirname = self.get_target_private_dir_abs(t)
             os.makedirs(priv_dirname, exist_ok=True)
 
-    def get_compiler_for_lang(self, lang):
-        for i in self.build.compilers:
-            if i.language == lang:
-                return i
-        raise RuntimeError('No compiler for language ' + lang)
-
-    def get_compiler_for_source(self, src, is_cross):
-        comp = self.build.cross_compilers if is_cross else self.build.compilers
-        for i in comp:
-            if i.can_compile(src):
-                return i
-        if isinstance(src, mesonlib.File):
-            src = src.fname
-        raise RuntimeError('No specified compiler can handle file ' + src)
-
-    def get_target_filename(self, target):
-        assert(isinstance(target, (build.BuildTarget, build.CustomTarget)))
-        targetdir = self.get_target_dir(target)
-        fname = target.get_filename()
-        if isinstance(fname, list):
-            # FIXME FIXME FIXME: build.CustomTarget has multiple output files
-            # and get_filename() returns them all
-            fname = fname[0]
-        filename = os.path.join(targetdir, fname)
-        return filename
+    def get_target_filename(self, t):
+        if isinstance(t, build.CustomTarget):
+            if len(t.get_outputs()) != 1:
+                mlog.warning('custom_target {!r} has more than one output! ' \
+                             'Using the first one.'.format(t.name))
+            filename = t.get_outputs()[0]
+        else:
+            assert(isinstance(t, build.BuildTarget))
+            filename = t.get_filename()
+        return os.path.join(self.get_target_dir(t), filename)
 
     def get_target_filename_abs(self, target):
         return os.path.join(self.environment.get_build_dir(), self.get_target_filename(target))
@@ -140,14 +125,30 @@ class Backend():
         dirname = os.path.join(self.environment.get_build_dir(), self.get_target_private_dir(target))
         return dirname
 
+    def get_target_generated_dir(self, target, gensrc, src):
+        """
+        Takes a BuildTarget, a generator source (CustomTarget or GeneratedList),
+        and a generated source filename.
+        Returns the full path of the generated source relative to the build root
+        """
+        # CustomTarget generators output to the build dir of the CustomTarget
+        if isinstance(gensrc, build.CustomTarget):
+            return os.path.join(self.get_target_dir(gensrc), src)
+        # GeneratedList generators output to the private build directory of the
+        # target that the GeneratedList is used in
+        return os.path.join(self.get_target_private_dir(target), src)
+
+    def get_unity_source_filename(self, target, suffix):
+        return target.name + '-unity.' + suffix
+
     def generate_unity_files(self, target, unity_src):
-        langlist = {}
         abs_files = []
         result = []
+        compsrcs = classify_unity_sources(target.compilers.values(), unity_src)
 
-        def init_language_file(language, suffix):
+        def init_language_file(suffix):
             outfilename = os.path.join(self.get_target_private_dir_abs(target),
-                                       target.name + '-unity' + suffix)
+                                       self.get_unity_source_filename(target, suffix))
             outfileabs = os.path.join(self.environment.get_build_dir(),
                                       outfilename)
             outfileabs_tmp = outfileabs + '.tmp'
@@ -158,20 +159,12 @@ class Backend():
             result.append(outfilename)
             return open(outfileabs_tmp, 'w')
 
-        try:
-            for src in unity_src:
-                comp = self.get_compiler_for_source(src, target.is_cross)
-                language = comp.get_language()
-                try:
-                    ofile = langlist[language]
-                except KeyError:
-                    suffix = '.' + comp.get_default_suffix()
-                    ofile = langlist[language] = init_language_file(language,
-                                                                    suffix)
-                ofile.write('#include<%s>\n' % src)
-        finally:
-            for x in langlist.values():
-                x.close()
+        # For each language, generate a unity source file and return the list
+        for comp, srcs in compsrcs.items():
+            lang = comp.get_language()
+            with init_language_file(comp.get_default_suffix()) as ofile:
+                for src in srcs:
+                    ofile.write('#include<%s>\n' % src)
         [mesonlib.replace_if_different(x, x + '.tmp') for x in abs_files]
         return result
 
@@ -207,11 +200,15 @@ class Backend():
         with open(exe_data, 'wb') as f:
             if isinstance(exe, dependencies.ExternalProgram):
                 exe_fullpath = exe.fullpath
+                exe_needs_wrapper = False
             elif isinstance(exe, (build.BuildTarget, build.CustomTarget)):
                 exe_fullpath = [self.get_target_filename_abs(exe)]
+                exe_needs_wrapper = exe.is_cross
             else:
                 exe_fullpath = [exe]
-            is_cross = self.environment.is_cross_build() and \
+                exe_needs_wrapper = False
+            is_cross = exe_needs_wrapper and \
+                self.environment.is_cross_build() and \
                 self.environment.cross_info.need_cross_compiler() and \
                 self.environment.cross_info.need_exe_wrapper()
             if is_cross:
@@ -239,19 +236,23 @@ class Backend():
 
     def determine_linker(self, target, src):
         if isinstance(target, build.StaticLibrary):
-            if self.build.static_cross_linker is not None:
+            if target.is_cross:
                 return self.build.static_cross_linker
             else:
                 return self.build.static_linker
-        if len(self.build.compilers) == 1:
-            return self.build.compilers[0]
+        if target.is_cross:
+            compilers = self.build.cross_compilers
+        else:
+            compilers = self.build.compilers
+        if len(compilers) == 1:
+            return compilers[0]
         # Currently a bit naive. C++ must
         # be linked with a C++ compiler, but
         # otherwise we don't care. This will
         # become trickier if and when Fortran
         # and the like become supported.
         cpp = None
-        for c in self.build.compilers:
+        for c in compilers:
             if c.get_language() == 'cpp':
                 cpp = c
                 break
@@ -259,30 +260,46 @@ class Backend():
             for s in src:
                 if c.can_compile(s):
                     return cpp
-        for c in self.build.compilers:
+        for c in compilers:
             if c.get_language() == 'vala':
                 continue
             for s in src:
                 if c.can_compile(s):
                     return c
-        raise RuntimeError('Unreachable code')
+        raise AssertionError("BUG: Couldn't determine linker for sources {!r}".format(src))
 
     def object_filename_from_source(self, target, source):
-        return source.fname.replace('/', '_').replace('\\', '_') + '.' + self.environment.get_object_suffix()
+        if isinstance(source, mesonlib.File):
+            source = source.fname
+        # foo.vala files compile down to foo.c and then foo.c.o, not foo.vala.o
+        if source.endswith('.vala'):
+            source = os.path.join(self.get_target_private_dir(target), source[:-5] + '.c')
+        return source.replace('/', '_').replace('\\', '_') + '.' + self.environment.get_object_suffix()
 
-    def determine_ext_objs(self, extobj, proj_dir_to_build_root=''):
+    def determine_ext_objs(self, extobj, proj_dir_to_build_root):
         result = []
         targetdir = self.get_target_private_dir(extobj.target)
+        # With unity builds, there's just one object that contains all the
+        # sources, so if we want all the objects, just return that.
+        if self.environment.coredata.get_builtin_option('unity'):
+            if not extobj.unity_compatible:
+                # This should never happen
+                msg = 'BUG: Meson must not allow extracting single objects ' \
+                      'in Unity builds'
+                raise AssertionError(msg)
+            comp = get_compiler_for_source(extobj.target.compilers.values(),
+                                           extobj.srclist[0])
+            # The unity object name uses the full absolute path of the source file
+            osrc = os.path.join(self.get_target_private_dir_abs(extobj.target),
+                                self.get_unity_source_filename(extobj.target,
+                                                               comp.get_default_suffix()))
+            objname = self.object_filename_from_source(extobj.target, osrc)
+            objpath = os.path.join(proj_dir_to_build_root, targetdir, objname)
+            return [objpath]
         for osrc in extobj.srclist:
-            # If extracting in a subproject, the subproject
-            # name gets duplicated in the file name.
-            pathsegs = osrc.subdir.split(os.sep)
-            if pathsegs[0] == 'subprojects':
-                pathsegs = pathsegs[2:]
-            fixedpath = os.sep.join(pathsegs)
-            objname = os.path.join(proj_dir_to_build_root, targetdir,
-                                   self.object_filename_from_source(extobj.target, osrc))
-            result.append(objname)
+            objname = self.object_filename_from_source(extobj.target, osrc)
+            objpath = os.path.join(proj_dir_to_build_root, targetdir, objname)
+            result.append(objpath)
         return result
 
     def get_pch_include_args(self, compiler, target):
@@ -327,13 +344,17 @@ class Backend():
                 extra_args.append(arg)
         return extra_args
 
-    def generate_basic_compiler_args(self, target, compiler):
+    def generate_basic_compiler_args(self, target, compiler, no_warn_args=False):
         commands = []
         commands += self.get_cross_stdlib_args(target, compiler)
         commands += compiler.get_always_args()
-        commands += compiler.get_warn_args(self.environment.coredata.get_builtin_option('warning_level'))
+        if no_warn_args:
+            commands += compiler.get_no_warn_args()
+        else:
+            commands += compiler.get_warn_args(self.environment.coredata.get_builtin_option('warning_level'))
         commands += compiler.get_option_compile_args(self.environment.coredata.compiler_options)
         commands += self.build.get_global_args(compiler)
+        commands += self.build.get_project_args(compiler, target.subproject)
         commands += self.environment.coredata.external_args[compiler.get_language()]
         commands += self.escape_extra_args(compiler, target.get_extra_args(compiler.get_language()))
         commands += compiler.get_buildtype_args(self.environment.coredata.get_builtin_option('buildtype'))
@@ -341,6 +362,8 @@ class Backend():
             commands += compiler.get_werror_args()
         if isinstance(target, build.SharedLibrary):
             commands += compiler.get_pic_args()
+        if isinstance(target, build.StaticLibrary) and target.pic:
+            commands += compiler.get_pic_args()
         for dep in target.get_external_deps():
             # Cflags required by external deps might have UNIX-specific flags,
             # so filter them out if needed
@@ -515,15 +538,17 @@ class Backend():
             outdir = '.'
         if absolute_paths:
             outdir = os.path.join(self.environment.get_build_dir(), outdir)
-        for i in target.sources:
+        for i in target.get_sources():
             if hasattr(i, 'held_object'):
                 i = i.held_object
             if isinstance(i, str):
                 fname = [os.path.join(self.build_to_src, target.subdir, i)]
-            elif isinstance(i, (build.BuildTarget, build.CustomTarget)):
+            elif isinstance(i, build.BuildTarget):
                 fname = [self.get_target_filename(i)]
+            elif isinstance(i, build.CustomTarget):
+                fname = [os.path.join(self.get_target_dir(i), p) for p in i.get_outputs()]
             elif isinstance(i, build.GeneratedList):
-                fname = [os.path.join(self.get_target_private_dir(target), p) for p in i.get_outfilelist()]
+                fname = [os.path.join(self.get_target_private_dir(target), p) for p in i.get_outputs()]
             else:
                 fname = [i.rel_to_builddir(self.build_to_src)]
             if absolute_paths:
@@ -537,7 +562,7 @@ class Backend():
             elif isinstance(i, build.CustomTarget):
                 # GIR scanner will attempt to execute this binary but
                 # it assumes that it is in path, so always give it a full path.
-                tmp = i.get_filename()[0]
+                tmp = i.get_outputs()[0]
                 i = os.path.join(self.get_target_dir(i), tmp)
             elif isinstance(i, mesonlib.File):
                 i = i.rel_to_builddir(self.build_to_src)
@@ -551,32 +576,52 @@ class Backend():
                 i = i.replace('@INPUT%d@' % j, src)
             for (j, res) in enumerate(ofilenames):
                 i = i.replace('@OUTPUT%d@' % j, res)
-            if i == '@INPUT@':
-                cmd += srcs
-            elif i == '@OUTPUT@':
-                cmd += ofilenames
-            else:
-                if '@OUTDIR@' in i:
-                    i = i.replace('@OUTDIR@', outdir)
-                elif '@DEPFILE@' in i:
-                    if target.depfile is None:
-                        raise MesonException('Custom target %s has @DEPFILE@ but no depfile keyword argument.' % target.name)
-                    if absolute_paths:
-                        dfilename = os.path.join(self.get_target_private_dir_abs(target), target.depfile)
-                    else:
-                        dfilename = os.path.join(self.get_target_private_dir(target), target.depfile)
-                    i = i.replace('@DEPFILE@', dfilename)
-                elif '@PRIVATE_OUTDIR_' in i:
-                    match = re.search('@PRIVATE_OUTDIR_(ABS_)?([-a-zA-Z0-9.@:]*)@', i)
-                    source = match.group(0)
-                    if match.group(1) is None and not absolute_paths:
-                        lead_dir = ''
-                    else:
-                        lead_dir = self.environment.get_build_dir()
-                    i = i.replace(source,
-                                  os.path.join(lead_dir,
-                                               outdir))
-                cmd.append(i)
+            if '@INPUT@' in i:
+                msg = 'Custom target {} has @INPUT@ in the command, but'.format(target.name)
+                if len(srcs) == 0:
+                    raise MesonException(msg + ' no input files')
+                if i == '@INPUT@':
+                    cmd += srcs
+                    continue
+                else:
+                    if len(srcs) > 1:
+                        raise MesonException(msg + ' more than one input file')
+                    i = i.replace('@INPUT@', srcs[0])
+            elif '@OUTPUT@' in i:
+                msg = 'Custom target {} has @OUTPUT@ in the command, but'.format(target.name)
+                if len(ofilenames) == 0:
+                    raise MesonException(msg + ' no output files')
+                if i == '@OUTPUT@':
+                    cmd += ofilenames
+                    continue
+                else:
+                    if len(ofilenames) > 1:
+                        raise MesonException(msg + ' more than one output file')
+                    i = i.replace('@OUTPUT@', ofilenames[0])
+            elif '@OUTDIR@' in i:
+                i = i.replace('@OUTDIR@', outdir)
+            elif '@DEPFILE@' in i:
+                if target.depfile is None:
+                    msg = 'Custom target {!r} has @DEPFILE@ but no depfile ' \
+                          'keyword argument.'.format(target.name)
+                    raise MesonException(msg)
+                dfilename = os.path.join(outdir, target.depfile)
+                i = i.replace('@DEPFILE@', dfilename)
+            elif '@PRIVATE_OUTDIR_' in i:
+                match = re.search('@PRIVATE_OUTDIR_(ABS_)?([^\/\s*]*)@', i)
+                if not match:
+                    msg = 'Custom target {!r} has an invalid argument {!r}' \
+                          ''.format(target.name, i)
+                    raise MesonException(msg)
+                source = match.group(0)
+                if match.group(1) is None and not absolute_paths:
+                    lead_dir = ''
+                else:
+                    lead_dir = self.environment.get_build_dir()
+                i = i.replace(source,
+                              os.path.join(lead_dir,
+                                           outdir))
+            cmd.append(i)
         # This should not be necessary but removing it breaks
         # building GStreamer on Windows. The underlying issue
         # is problems with quoting backslashes on Windows
