@@ -17,7 +17,7 @@ from . import environment
 from . import dependencies
 from . import mlog
 import copy, os, re
-from .mesonlib import File, flatten, MesonException, stringlistify
+from .mesonlib import File, flatten, MesonException, stringlistify, classify_unity_sources
 from .environment import for_windows, for_darwin
 
 known_basic_kwargs = {'install' : True,
@@ -44,12 +44,22 @@ known_basic_kwargs = {'install' : True,
                       'native' : True,
                      }
 
-known_shlib_kwargs = known_basic_kwargs.copy()
-known_shlib_kwargs.update({'version' : True,
-                           'soversion' : True,
-                           'name_prefix' : True,
-                           'name_suffix' : True,
-                           'vs_module_defs' : True})
+# These contain kwargs supported by both static and shared libraries. These are
+# combined here because a library() call might be shared_library() or
+# static_library() at runtime based on the configuration.
+# FIXME: Find a way to pass that info down here so we can have proper target
+# kwargs checking when specifically using shared_library() or static_library().
+known_lib_kwargs = known_basic_kwargs.copy()
+known_lib_kwargs.update({'version' : True, # Only for shared libs
+                         'soversion' : True, # Only for shared libs
+                         'name_prefix' : True,
+                         'name_suffix' : True,
+                         'vs_module_defs' : True, # Only for shared libs
+                         'vala_header': True,
+                         'vala_vapi': True,
+                         'vala_gir' : True,
+                         'pic' : True, # Only for static libs
+                        })
 
 def compilers_are_msvc(compilers):
     """
@@ -80,7 +90,9 @@ class Build:
         self.compilers = []
         self.cross_compilers = []
         self.global_args = {}
+        self.projects_args = {}
         self.global_link_args = {}
+        self.projects_link_args = {}
         self.tests = []
         self.benchmarks = []
         self.headers = []
@@ -144,9 +156,22 @@ class Build:
     def get_global_args(self, compiler):
         return self.global_args.get(compiler.get_language(), [])
 
+    def get_project_args(self, compiler, project):
+        args = self.projects_args.get(project)
+        if not args:
+            return []
+        return args.get(compiler.get_language(), [])
+
     def get_global_link_args(self, compiler):
         return self.global_link_args.get(compiler.get_language(), [])
 
+    def get_project_link_args(self, compiler, project):
+        link_args = self.projects_link_args.get(project)
+        if not link_args:
+            return []
+
+        return link_args.get(compiler.get_language(), [])
+
 class IncludeDirs():
     def __init__(self, curdir, dirs, is_system, extra_build_dirs=None):
         self.curdir = curdir
@@ -169,9 +194,42 @@ class IncludeDirs():
         return self.extra_build_dirs
 
 class ExtractedObjects():
+    '''
+    Holds a list of sources for which the objects must be extracted
+    '''
     def __init__(self, target, srclist):
         self.target = target
         self.srclist = srclist
+        self.check_unity_compatible()
+
+    def check_unity_compatible(self):
+        # Figure out if the extracted object list is compatible with a Unity
+        # build. When we're doing a Unified build, we go through the sources,
+        # and create a single source file from each subset of the sources that
+        # can be compiled with a specific compiler. Then we create one object
+        # from each unified source file.
+        # If the list of sources for which we want objects is the same as the
+        # list of sources that go into each unified build, we're good.
+        self.unity_compatible = False
+        srclist_set = set(self.srclist)
+        # Objects for all the sources are required, so we're compatible
+        if srclist_set == set(self.target.sources):
+            self.unity_compatible = True
+            return
+        # Check if the srclist is a subset (of the target's sources) that is
+        # going to form a unified source file and a single object
+        compsrcs = classify_unity_sources(self.target.compilers.values(),
+                                          self.target.sources)
+        for srcs in compsrcs.values():
+            if srclist_set == set(srcs):
+                self.unity_compatible = True
+                return
+        msg = 'Single object files can not be extracted in Unity builds. ' \
+              'You can only extract all the object files at once.'
+        raise MesonException(msg)
+
+    def get_want_all_objects(self):
+        return self.want_all_objects
 
 class EnvironmentVariables():
     def __init__(self):
@@ -222,6 +280,8 @@ class BuildTarget():
         self.include_dirs = []
         self.link_targets = []
         self.link_depends = []
+        self.name_prefix_set = False
+        self.name_suffix_set = False
         self.filename = 'no_name'
         # The file with debugging symbols
         self.debug_filename = None
@@ -265,8 +325,8 @@ class BuildTarget():
             if not k in known_kwargs:
                 unknowns.append(k)
         if len(unknowns) > 0:
-            mlog.log(mlog.bold('Warning:'), 'Unknown keyword argument(s) in target %s: %s.' %
-                     (self.name, ', '.join(unknowns)))
+            mlog.warning('Unknown keyword argument(s) in target %s: %s.' %
+                         (self.name, ', '.join(unknowns)))
 
     def process_objectlist(self, objects):
         assert(isinstance(objects, list))
@@ -303,6 +363,13 @@ class BuildTarget():
                 msg = 'Bad source of type {!r} in target {!r}.'.format(type(s).__name__, self.name)
                 raise InvalidArguments(msg)
 
+    @staticmethod
+    def can_compile_sources(compiler, sources):
+        for s in sources:
+            if compiler.can_compile(s):
+                return True
+        return False
+
     @staticmethod
     def can_compile_remove_sources(compiler, sources):
         removed = False
@@ -313,16 +380,23 @@ class BuildTarget():
         return removed
 
     def process_compilers(self):
-        if len(self.sources) == 0:
+        if len(self.sources) + len(self.generated) == 0:
             return
         sources = list(self.sources)
+        for gensrc in self.generated:
+            sources += gensrc.get_outputs()
+        # Populate list of compilers
         if self.is_cross:
             compilers = self.environment.coredata.cross_compilers
         else:
             compilers = self.environment.coredata.compilers
         for lang, compiler in compilers.items():
-            if self.can_compile_remove_sources(compiler, sources):
+            if self.can_compile_sources(compiler, sources):
                 self.compilers[lang] = compiler
+        # If all our sources are Vala, our target also needs the C compiler but
+        # it won't get added above.
+        if 'vala' in self.compilers and 'c' not in self.compilers:
+            self.compilers['c'] = compilers['c']
 
     def validate_sources(self):
         if len(self.sources) == 0:
@@ -374,18 +448,15 @@ class BuildTarget():
         if 'link_with' in self.kwargs:
             self.kwargs['link_with'] = self.unpack_holder(self.kwargs['link_with'])
 
-    def extract_objects(self, srcargs):
+    def extract_objects(self, srclist):
         obj_src = []
-        for srclist in srcargs:
-            if not isinstance(srclist, list):
-                srclist = [srclist]
-            for src in srclist:
-                if not isinstance(src, str):
-                    raise MesonException('Extraction arguments must be strings.')
-                src = File(False, self.subdir, src)
-                if src not in self.sources:
-                    raise MesonException('Tried to extract unknown source %s.' % src)
-                obj_src.append(src)
+        for src in srclist:
+            if not isinstance(src, str):
+                raise MesonException('Object extraction arguments must be strings.')
+            src = File(False, self.subdir, src)
+            if src not in self.sources:
+                raise MesonException('Tried to extract unknown source %s.' % src)
+            obj_src.append(src)
         return ExtractedObjects(self, obj_src)
 
     def extract_all_objects(self):
@@ -440,6 +511,10 @@ class BuildTarget():
         if not isinstance(valalist, list):
             valalist = [valalist]
         self.add_compiler_args('vala', valalist)
+        if not isinstance(self, Executable):
+            self.vala_header = kwargs.get('vala_header', self.name + '.h')
+            self.vala_vapi = kwargs.get('vala_vapi', self.name + '.vapi')
+        self.vala_gir = kwargs.get('vala_gir', None)
         dlist = stringlistify(kwargs.get('d_args', []))
         self.add_compiler_args('d', dlist)
         self.link_args = kwargs.get('link_args', [])
@@ -461,7 +536,7 @@ class BuildTarget():
         deplist = kwargs.get('dependencies', [])
         if not isinstance(deplist, list):
             deplist = [deplist]
-        self.add_external_deps(deplist)
+        self.add_deps(deplist)
         self.custom_install_dir = kwargs.get('install_dir', None)
         if self.custom_install_dir is not None:
             if not isinstance(self.custom_install_dir, str):
@@ -503,19 +578,34 @@ class BuildTarget():
             name_prefix = kwargs['name_prefix']
             if isinstance(name_prefix, list):
                 if len(name_prefix) != 0:
-                    raise InvalidArguments('Array must be empty to signify null.')
+                    raise InvalidArguments('name_prefix array must be empty to signify null.')
             elif not isinstance(name_prefix, str):
-                raise InvalidArguments('Name prefix must be a string.')
+                raise InvalidArguments('name_prefix must be a string.')
             self.prefix = name_prefix
+            self.name_prefix_set = True
         if 'name_suffix' in kwargs:
             name_suffix = kwargs['name_suffix']
             if isinstance(name_suffix, list):
                 if len(name_suffix) != 0:
-                    raise InvalidArguments('Array must be empty to signify null.')
+                    raise InvalidArguments('name_suffix array must be empty to signify null.')
             else:
                 if not isinstance(name_suffix, str):
-                    raise InvalidArguments('Name suffix must be a string.')
+                    raise InvalidArguments('name_suffix must be a string.')
                 self.suffix = name_suffix
+                self.name_suffix_set = True
+        if isinstance(self, StaticLibrary):
+            # You can't disable PIC on OS X. The compiler ignores -fno-PIC.
+            # PIC is always on for Windows (all code is position-independent
+            # since library loading is done differently)
+            if for_darwin(self.is_cross, self.environment) or for_windows(self.is_cross, self.environment):
+                self.pic = True
+            elif '-fPIC' in clist + cpplist:
+                mlog.warning("Use the 'pic' kwarg instead of passing -fPIC manually to static library {!r}".format(self.name))
+                self.pic = True
+            else:
+                self.pic = kwargs.get('pic', False)
+                if not isinstance(self.pic, bool):
+                    raise InvalidArguments('Argument pic to static library {!r} must be boolean'.format(self.name))
 
     def get_subdir(self):
         return self.subdir
@@ -523,6 +613,9 @@ class BuildTarget():
     def get_filename(self):
         return self.filename
 
+    def get_outputs(self):
+        return [self.filename]
+
     def get_debug_filename(self):
         """
         The name of the file that contains debugging symbols for this target
@@ -573,7 +666,7 @@ class BuildTarget():
     def get_include_dirs(self):
         return self.include_dirs
 
-    def add_external_deps(self, deps):
+    def add_deps(self, deps):
         if not isinstance(deps, list):
             deps = [deps]
         for dep in deps:
@@ -593,7 +686,7 @@ class BuildTarget():
                                                           [], [], [])
                 self.external_deps.append(extpart)
                 # Deps of deps.
-                self.add_external_deps(dep.ext_deps)
+                self.add_deps(dep.ext_deps)
             elif isinstance(dep, dependencies.Dependency):
                 self.external_deps.append(dep)
                 self.process_sourcelist(dep.get_sources())
@@ -602,10 +695,13 @@ class BuildTarget():
                 # about the interpreter so we can't import it and use isinstance.
                 # This should be reliable enough.
                 if hasattr(dep, 'subproject'):
-                    raise InvalidArguments('''Tried to use subproject object as a dependency.
-You probably wanted to use a dependency declared in it instead. Access it
-by calling get_variable() on the subproject object.''')
-                raise InvalidArguments('Argument is not an external dependency.')
+                    raise InvalidArguments('Tried to use subproject object as a dependency.\n'
+                            'You probably wanted to use a dependency declared in it instead.\n'
+                            'Access it by calling get_variable() on the subproject object.')
+                raise InvalidArguments('Argument is of an unacceptable type {!r}.\nMust be '
+                        'either an external dependency (returned by find_library() or '
+                        'dependency()) or an internal dependency (returned by '
+                        'declare_dependency()).'.format(type(dep).__name__))
 
     def get_external_deps(self):
         return self.external_deps
@@ -617,23 +713,21 @@ by calling get_variable() on the subproject object.''')
             if hasattr(t, 'held_object'):
                 t = t.held_object
             if not isinstance(t, (StaticLibrary, SharedLibrary)):
-                raise InvalidArguments('Link target is not library.')
+                raise InvalidArguments('Link target {!r} is not library.'.format(t.name))
+            if isinstance(self, SharedLibrary) and isinstance(t, StaticLibrary) and not t.pic:
+                msg = "Can't link non-PIC static library {!r} into shared library {!r}. ".format(t.name, self.name)
+                msg += "Use the 'pic' option to static_library to build with PIC."
+                raise InvalidArguments(msg)
             if self.is_cross != t.is_cross:
-                raise InvalidArguments('Tried to mix cross built and native libraries in target %s.' % self.name)
+                raise InvalidArguments('Tried to mix cross built and native libraries in target {!r}'.format(self.name))
             self.link_targets.append(t)
 
-    def set_generated(self, genlist):
-        for g in genlist:
-            if not(isinstance(g, GeneratedList)):
-                raise InvalidArguments('Generated source argument is not the output of a generator.')
-            self.generated.append(g)
-
     def add_pch(self, language, pchlist):
         if len(pchlist) == 0:
             return
         elif len(pchlist) == 1:
             if not environment.is_header(pchlist[0]):
-                raise InvalidArguments('Pch argument %s is not a header.' % pchlist[0])
+                raise InvalidArguments('PCH argument %s is not a header.' % pchlist[0])
         elif len(pchlist) == 2:
             if environment.is_header(pchlist[0]):
                 if not environment.is_source(pchlist[1]):
@@ -676,8 +770,7 @@ by calling get_variable() on the subproject object.''')
 class Generator():
     def __init__(self, args, kwargs):
         if len(args) != 1:
-            raise InvalidArguments('Generator requires one and only one positional argument')
-        
+            raise InvalidArguments('Generator requires exactly one positional argument: the executable')
         exe = args[0]
         if hasattr(exe, 'held_object'):
             exe = exe.held_object
@@ -733,14 +826,14 @@ class Generator():
 
     def get_base_outnames(self, inname):
         plainname = os.path.split(inname)[1]
-        basename = plainname.split('.')[0]
+        basename = os.path.splitext(plainname)[0]
         return [x.replace('@BASENAME@', basename).replace('@PLAINNAME@', plainname) for x in self.outputs]
 
     def get_dep_outname(self, inname):
         if self.depfile is None:
             raise InvalidArguments('Tried to get dep name for rule that does not have dependency file defined.')
         plainname = os.path.split(inname)[1]
-        basename = plainname.split('.')[0]
+        basename = os.path.splitext(plainname)[0]
         return self.depfile.replace('@BASENAME@', basename).replace('@PLAINNAME@', plainname)
 
     def get_arglist(self):
@@ -751,6 +844,7 @@ class GeneratedList():
         if hasattr(generator, 'held_object'):
             generator = generator.held_object
         self.generator = generator
+        self.name = self.generator.exe
         self.infilelist = []
         self.outfilelist = []
         self.outmap = {}
@@ -763,10 +857,10 @@ class GeneratedList():
         self.outfilelist += outfiles
         self.outmap[newfile] = outfiles
 
-    def get_infilelist(self):
+    def get_inputs(self):
         return self.infilelist
 
-    def get_outfilelist(self):
+    def get_outputs(self):
         return self.outfilelist
 
     def get_outputs_for(self, filename):
@@ -804,6 +898,8 @@ class Executable(BuildTarget):
 
 class StaticLibrary(BuildTarget):
     def __init__(self, name, subdir, subproject, is_cross, sources, objects, environment, kwargs):
+        if 'pic' not in kwargs and 'b_staticpic' in environment.coredata.base_options:
+            kwargs['pic'] = environment.coredata.base_options['b_staticpic'].value
         super().__init__(name, subdir, subproject, is_cross, sources, objects, environment, kwargs)
         if 'cs' in self.compilers:
             raise InvalidArguments('Static libraries not supported for C#.')
@@ -831,6 +927,9 @@ class StaticLibrary(BuildTarget):
     def type_suffix(self):
         return "@sta"
 
+    def check_unknown_kwargs(self, kwargs):
+        self.check_unknown_kwargs_int(kwargs, known_lib_kwargs)
+
 class SharedLibrary(BuildTarget):
     def __init__(self, name, subdir, subproject, is_cross, sources, objects, environment, kwargs):
         self.soversion = None
@@ -988,7 +1087,7 @@ class SharedLibrary(BuildTarget):
             self.link_depends.append(path)
 
     def check_unknown_kwargs(self, kwargs):
-        self.check_unknown_kwargs_int(kwargs, known_shlib_kwargs)
+        self.check_unknown_kwargs_int(kwargs, known_lib_kwargs)
 
     def get_import_filename(self):
         """
@@ -1052,14 +1151,13 @@ class CustomTarget:
         self.depfile = None
         self.process_kwargs(kwargs)
         self.extra_files = []
-        self.install_rpath = ''
         unknowns = []
         for k in kwargs:
             if k not in CustomTarget.known_kwargs:
                 unknowns.append(k)
         if len(unknowns) > 0:
-            mlog.log(mlog.bold('Warning:'), 'Unknown keyword arguments in target %s: %s' %
-                     (self.name, ', '.join(unknowns)))
+            mlog.warning('Unknown keyword arguments in target %s: %s' %
+                         (self.name, ', '.join(unknowns)))
 
     def __repr__(self):
         repr_str = "<{0} {1}: {2}>"
@@ -1094,8 +1192,7 @@ class CustomTarget:
                 raise InvalidArguments('Output must not contain a path segment.')
         self.capture = kwargs.get('capture', False)
         if self.capture and len(self.output) != 1:
-            raise InvalidArguments(
-                'Capturing can only output to a single file.')
+            raise InvalidArguments('Capturing can only output to a single file.')
         if 'command' not in kwargs:
             raise InvalidArguments('Missing keyword argument "command".')
         if 'depfile' in kwargs:
@@ -1116,7 +1213,7 @@ class CustomTarget:
                 final_cmd.append(c)
             elif isinstance(c, dependencies.ExternalProgram):
                 if not c.found():
-                    raise InvalidArguments('Tried to use not found external program in a build rule.')
+                    raise InvalidArguments('Tried to use not found external program {!r} in a build rule.'.format(c.name))
                 final_cmd += c.get_command()
             elif isinstance(c, (BuildTarget, CustomTarget)):
                 self.dependencies.append(c)
@@ -1157,7 +1254,7 @@ class CustomTarget:
             while hasattr(ed, 'held_object'):
                 ed = ed.held_object
             if not isinstance(ed, (CustomTarget, BuildTarget)):
-                raise InvalidArguments('Can only depend on toplevel targets.')
+                raise InvalidArguments('Can only depend on toplevel targets: custom_target or build_target (executable or a library)')
             self.extra_depends.append(ed)
         depend_files = kwargs.get('depend_files', [])
         if not isinstance(depend_files, list):
@@ -1167,7 +1264,7 @@ class CustomTarget:
                 self.depend_files.append(i)
             else:
                 mlog.debug(i)
-                raise InvalidArguments('Unknown type in depend_files.')
+                raise InvalidArguments('Unknown type {!r} in depend_files.'.format(type(i).__name__))
 
     def get_basename(self):
         return self.name
@@ -1184,11 +1281,11 @@ class CustomTarget:
     def get_subdir(self):
         return self.subdir
 
-    def get_filename(self):
+    def get_outputs(self):
         return self.output
 
-    def get_aliaslist(self):
-        return []
+    def get_filename(self):
+        return self.output[0]
 
     def get_sources(self):
         return self.sources
