@@ -22,19 +22,25 @@
 import re
 import os, stat, glob, subprocess, shutil
 import sysconfig
+from collections import OrderedDict
 from . mesonlib import MesonException
 from . import mlog
 from . import mesonlib
-from .environment import detect_cpu_family
+from .environment import detect_cpu_family, for_windows
 
 class DependencyException(MesonException):
     def __init__(self, *args, **kwargs):
         MesonException.__init__(self, *args, **kwargs)
 
 class Dependency():
-    def __init__(self):
+    def __init__(self, type_name='unknown'):
         self.name = "null"
         self.is_found = False
+        self.type_name = type_name
+
+    def __repr__(self):
+        s = '<{0} {1}: {2}>'
+        return s.format(self.__class__.__name__, self.name, self.is_found)
 
     def get_compile_args(self):
         return []
@@ -59,9 +65,15 @@ class Dependency():
     def need_threads(self):
         return False
 
+    def type_name(self):
+        return self.type_name
+
+    def get_pkgconfig_variable(self, variable_name):
+        raise MesonException('Tried to get a pkg-config variable from a non-pkgconfig dependency.')
+
 class InternalDependency(Dependency):
     def __init__(self, version, incdirs, compile_args, link_args, libraries, sources, ext_deps):
-        super().__init__()
+        super().__init__('internal')
         self.version = version
         self.include_directories = incdirs
         self.compile_args = compile_args
@@ -83,10 +95,11 @@ class PkgConfigDependency(Dependency):
     pkgconfig_found = None
 
     def __init__(self, name, environment, kwargs):
-        Dependency.__init__(self)
+        Dependency.__init__(self, 'pkgconfig')
         self.is_libtool = False
         self.required = kwargs.get('required', True)
         self.static = kwargs.get('static', False)
+        self.silent = kwargs.get('silent', False)
         if not isinstance(self.static, bool):
             raise DependencyException('Static keyword must be boolean')
         self.cargs = []
@@ -115,16 +128,12 @@ class PkgConfigDependency(Dependency):
 
         mlog.debug('Determining dependency %s with pkg-config executable %s.' % (name, pkgbin))
         self.pkgbin = pkgbin
-        p = subprocess.Popen([pkgbin, '--modversion', name],
-                             stdout=subprocess.PIPE,
-                             stderr=subprocess.PIPE)
-        out = p.communicate()[0]
-        if p.returncode != 0:
+        ret, self.modversion = self._call_pkgbin(['--modversion', name])
+        if ret != 0:
             if self.required:
                 raise DependencyException('%s dependency %s not found.' % (self.type_string, name))
             self.modversion = 'none'
             return
-        self.modversion = out.decode().strip()
         found_msg = ['%s dependency' % self.type_string, mlog.bold(name), 'found:']
         self.version_requirement = kwargs.get('version', None)
         if self.version_requirement is None:
@@ -136,40 +145,45 @@ class PkgConfigDependency(Dependency):
             if not self.is_found:
                 found_msg += [mlog.red('NO'), 'found {!r}'.format(self.modversion),
                               'but need {!r}'.format(self.version_requirement)]
-                mlog.log(*found_msg)
+                if not self.silent:
+                    mlog.log(*found_msg)
                 if self.required:
                     raise DependencyException(
                         'Invalid version of a dependency, needed %s %s found %s.' %
                         (name, self.version_requirement, self.modversion))
                 return
         found_msg += [mlog.green('YES'), self.modversion]
-        mlog.log(*found_msg)
+        if not self.silent:
+            mlog.log(*found_msg)
         # Fetch cargs to be used while using this dependency
         self._set_cargs()
         # Fetch the libraries and library paths needed for using this
         self._set_libs()
 
-    def _set_cargs(self):
-        p = subprocess.Popen([self.pkgbin, '--cflags', self.name],
-                             stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+    def _call_pkgbin(self, args):
+        p = subprocess.Popen([self.pkgbin] + args,
+                             stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+                             env=os.environ, universal_newlines=True)
         out = p.communicate()[0]
-        if p.returncode != 0:
+        return (p.returncode, out.strip())
+
+    def _set_cargs(self):
+        ret, out = self._call_pkgbin(['--cflags', self.name])
+        if ret != 0:
             raise DependencyException('Could not generate cargs for %s:\n\n%s' % \
                                       (self.name, out.decode(errors='ignore')))
-        self.cargs = out.decode().split()
+        self.cargs = out.split()
 
     def _set_libs(self):
-        libcmd = [self.pkgbin, '--libs']
+        libcmd = [self.name, '--libs']
         if self.static:
             libcmd.append('--static')
-        p = subprocess.Popen(libcmd + [self.name],
-                             stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-        out = p.communicate()[0]
-        if p.returncode != 0:
+        ret, out = self._call_pkgbin(libcmd)
+        if ret != 0:
             raise DependencyException('Could not generate libs for %s:\n\n%s' % \
                                       (self.name, out.decode(errors='ignore')))
         self.libs = []
-        for lib in out.decode().split():
+        for lib in out.split():
             if lib.endswith(".la"):
                 shared_libname = self.extract_libtool_shlib(lib)
                 shared_lib = os.path.join(os.path.dirname(lib), shared_libname)
@@ -184,19 +198,16 @@ class PkgConfigDependency(Dependency):
                 self.is_libtool = True
             self.libs.append(lib)
 
-    def get_variable(self, variable_name):
-        p = subprocess.Popen([self.pkgbin, '--variable=%s' % variable_name, self.name],
-                             stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-        out = p.communicate()[0]
+    def get_pkgconfig_variable(self, variable_name):
+        ret, out = self._call_pkgbin(['--variable=' + variable_name, self.name])
         variable = ''
-        if p.returncode != 0:
+        if ret != 0:
             if self.required:
                 raise DependencyException('%s dependency %s not found.' %
                                           (self.type_string, self.name))
         else:
-            variable = out.decode().strip()
-        mlog.debug('return of subprocess : %s' % variable)
-
+            variable = out.strip()
+        mlog.debug('Got pkgconfig variable %s : %s' % (variable_name, variable))
         return variable
 
     def get_modversion(self):
@@ -217,14 +228,16 @@ class PkgConfigDependency(Dependency):
                                  stderr=subprocess.PIPE)
             out = p.communicate()[0]
             if p.returncode == 0:
-                mlog.log('Found pkg-config:', mlog.bold(shutil.which('pkg-config')),
-                         '(%s)' % out.decode().strip())
+                if not self.silent:
+                    mlog.log('Found pkg-config:', mlog.bold(shutil.which('pkg-config')),
+                             '(%s)' % out.decode().strip())
                 PkgConfigDependency.pkgconfig_found = True
                 return
         except Exception:
             pass
         PkgConfigDependency.pkgconfig_found = False
-        mlog.log('Found Pkg-config:', mlog.red('NO'))
+        if not self.silent:
+            mlog.log('Found Pkg-config:', mlog.red('NO'))
 
     def found(self):
         return self.is_found
@@ -268,7 +281,7 @@ class WxDependency(Dependency):
     wx_found = None
 
     def __init__(self, environment, kwargs):
-        Dependency.__init__(self)
+        Dependency.__init__(self, 'wx')
         self.is_found = False
         if WxDependency.wx_found is None:
             self.check_wxconfig()
@@ -442,29 +455,37 @@ class ExternalProgram():
         return self.name
 
 class ExternalLibrary(Dependency):
-    def __init__(self, name, link_args=None, silent=False):
-        super().__init__()
+    # TODO: Add `lang` to the parent Dependency object so that dependencies can
+    # be expressed for languages other than C-like
+    def __init__(self, name, link_args=None, language=None, silent=False):
+        super().__init__('external')
         self.name = name
-        # Rename fullpath to link_args once standalone find_library() gets removed.
-        if link_args is not None:
-            if isinstance(link_args, list):
-                self.link_args = link_args
+        self.is_found = False
+        self.link_args = []
+        self.lang_args = []
+        if link_args:
+            self.is_found = True
+            if not isinstance(link_args, list):
+                link_args = [link_args]
+            if language:
+                self.lang_args = {language: link_args}
             else:
-                self.link_args = [link_args]
-        else:
-            self.link_args = link_args
+                self.link_args = link_args
         if not silent:
-            if self.found():
+            if self.is_found:
                 mlog.log('Library', mlog.bold(name), 'found:', mlog.green('YES'))
             else:
                 mlog.log('Library', mlog.bold(name), 'found:', mlog.red('NO'))
 
     def found(self):
-        return self.link_args is not None
+        return self.is_found
 
     def get_link_args(self):
-        if self.found():
-            return self.link_args
+        return self.link_args
+
+    def get_lang_args(self, lang):
+        if lang in self.lang_args:
+            return self.lang_args[lang]
         return []
 
 class BoostDependency(Dependency):
@@ -474,7 +495,7 @@ class BoostDependency(Dependency):
     name2lib = {'test' : 'unit_test_framework'}
 
     def __init__(self, environment, kwargs):
-        Dependency.__init__(self)
+        Dependency.__init__(self, 'boost')
         self.name = 'boost'
         self.environment = environment
         self.libdir = ''
@@ -675,18 +696,13 @@ class BoostDependency(Dependency):
 
 class GTestDependency(Dependency):
     def __init__(self, environment, kwargs):
-        Dependency.__init__(self)
+        Dependency.__init__(self, 'gtest')
         self.main = kwargs.get('main', False)
         self.name = 'gtest'
         self.libname = 'libgtest.so'
         self.libmain_name = 'libgtest_main.so'
         self.include_dir = '/usr/include'
-        self.src_include_dir = '/usr/src/gtest'
-        self.src_dir = '/usr/src/gtest/src'
-        self.all_src = mesonlib.File.from_absolute_file(
-            os.path.join(self.src_dir, 'gtest-all.cc'))
-        self.main_src = mesonlib.File.from_absolute_file(
-            os.path.join(self.src_dir, 'gtest_main.cc'))
+        self.src_dirs = ['/usr/src/gtest/src', '/usr/src/googletest/googletest/src']
         self.detect()
 
     def found(self):
@@ -709,7 +725,7 @@ class GTestDependency(Dependency):
                 self.link_args.append('-lgtest_main')
             self.sources = []
             mlog.log('Dependency GTest found:', mlog.green('YES'), '(prebuilt)')
-        elif os.path.exists(self.src_dir):
+        elif self.detect_srcdir():
             self.is_found = True
             self.compile_args = ['-I' + self.src_include_dir]
             self.link_args = []
@@ -723,11 +739,24 @@ class GTestDependency(Dependency):
             self.is_found = False
         return self.is_found
 
+    def detect_srcdir(self):
+        for s in self.src_dirs:
+            if os.path.exists(s):
+                self.src_dir = s
+                self.all_src = mesonlib.File.from_absolute_file(
+                    os.path.join(self.src_dir, 'gtest-all.cc'))
+                self.main_src = mesonlib.File.from_absolute_file(
+                    os.path.join(self.src_dir, 'gtest_main.cc'))
+                self.src_include_dir = os.path.normpath(os.path.join(self.src_dir, '..'))
+                return True
+        return False
+
     def get_compile_args(self):
         arr = []
         if self.include_dir != '/usr/include':
             arr.append('-I' + self.include_dir)
-        arr.append('-I' + self.src_include_dir)
+        if hasattr(self, 'src_include_dir'):
+            arr.append('-I' + self.src_include_dir)
         return arr
 
     def get_link_args(self):
@@ -742,7 +771,7 @@ class GTestDependency(Dependency):
 
 class GMockDependency(Dependency):
     def __init__(self, environment, kwargs):
-        Dependency.__init__(self)
+        Dependency.__init__(self, 'gmock')
         # GMock may be a library or just source.
         # Work with both.
         self.name = 'gmock'
@@ -760,12 +789,13 @@ class GMockDependency(Dependency):
             mlog.log('Dependency GMock found:', mlog.green('YES'), '(prebuilt)')
             return
 
-        for d in ['/usr/src/gmock/src', '/usr/src/gmock']:
+        for d in ['/usr/src/googletest/googlemock/src', '/usr/src/gmock/src', '/usr/src/gmock']:
             if os.path.exists(d):
                 self.is_found = True
                 # Yes, we need both because there are multiple
                 # versions of gmock that do different things.
-                self.compile_args = ['-I/usr/src/gmock', '-I/usr/src/gmock/src']
+                d2 = os.path.normpath(os.path.join(d, '..'))
+                self.compile_args = ['-I' + d, '-I' + d2]
                 self.link_args = []
                 all_src = mesonlib.File.from_absolute_file(os.path.join(d, 'gmock-all.cc'))
                 main_src = mesonlib.File.from_absolute_file(os.path.join(d, 'gmock_main.cc'))
@@ -794,11 +824,24 @@ class GMockDependency(Dependency):
     def found(self):
         return self.is_found
 
-class Qt5Dependency(Dependency):
-    def __init__(self, environment, kwargs):
-        Dependency.__init__(self)
-        self.name = 'qt5'
+class QtBaseDependency(Dependency):
+    def __init__(self, name, env, kwargs):
+        Dependency.__init__(self, name)
+        self.name = name
+        self.qtname = name.capitalize()
+        self.qtver = name[-1]
+        if self.qtver == "4":
+            self.qtpkgname = 'Qt'
+        else:
+            self.qtpkgname = self.qtname
         self.root = '/usr'
+        self.bindir = None
+        self.silent = kwargs.get('silent', False)
+        # We store the value of required here instead of passing it on to
+        # PkgConfigDependency etc because we want to try the qmake-based
+        # fallback as well.
+        self.required = kwargs.pop('required', True)
+        kwargs['required'] = False
         mods = kwargs.get('modules', [])
         self.cargs = []
         self.largs = []
@@ -806,69 +849,140 @@ class Qt5Dependency(Dependency):
         if isinstance(mods, str):
             mods = [mods]
         if len(mods) == 0:
-            raise DependencyException('No Qt5 modules specified.')
-        type_text = 'native'
-        if environment.is_cross_build() and kwargs.get('native', False):
-            type_text = 'cross'
-            self.pkgconfig_detect(mods, environment, kwargs)
-        elif not environment.is_cross_build() and shutil.which('pkg-config') is not None:
-            self.pkgconfig_detect(mods, environment, kwargs)
-        elif shutil.which('qmake') is not None:
-            self.qmake_detect(mods, kwargs)
-        else:
-            self.version = 'none'
+            raise DependencyException('No ' + self.qtname + '  modules specified.')
+        type_text = 'cross' if env.is_cross_build() else 'native'
+        found_msg = '{} {} {{}} dependency (modules: {}) found:' \
+                    ''.format(self.qtname, type_text, ', '.join(mods))
+        from_text = '`pkg-config`'
+        # Prefer pkg-config, then fallback to `qmake -query`
+        self._pkgconfig_detect(mods, env, kwargs)
         if not self.is_found:
-            mlog.log('Qt5 %s dependency found: ' % type_text, mlog.red('NO'))
-        else:
-            mlog.log('Qt5 %s dependency found: ' % type_text, mlog.green('YES'))
+            from_text = self._qmake_detect(mods, env, kwargs)
+            if not self.is_found:
+                # Reset compile args and link args
+                self.cargs = []
+                self.largs = []
+                from_text = '(checked pkg-config, qmake-{}, and qmake)' \
+                            ''.format(self.name)
+                self.version = 'none'
+                if self.required:
+                    err_msg = '{} {} dependency not found {}' \
+                              ''.format(self.qtname, type_text, from_text)
+                    raise DependencyException(err_msg)
+                if not self.silent:
+                    mlog.log(found_msg.format(from_text), mlog.red('NO'))
+                return
+            from_text = '`{}`'.format(from_text)
+        if not self.silent:
+            mlog.log(found_msg.format(from_text), mlog.green('YES'))
 
-    def pkgconfig_detect(self, mods, environment, kwargs):
-        modules = []
+    def compilers_detect(self):
+        "Detect Qt (4 or 5) moc, uic, rcc in the specified bindir or in PATH"
+        if self.bindir:
+            moc = ExternalProgram(os.path.join(self.bindir, 'moc'), silent=True)
+            uic = ExternalProgram(os.path.join(self.bindir, 'uic'), silent=True)
+            rcc = ExternalProgram(os.path.join(self.bindir, 'rcc'), silent=True)
+        else:
+            # We don't accept unsuffixed 'moc', 'uic', and 'rcc' because they
+            # are sometimes older, or newer versions.
+            moc = ExternalProgram('moc-' + self.name, silent=True)
+            uic = ExternalProgram('uic-' + self.name, silent=True)
+            rcc = ExternalProgram('rcc-' + self.name, silent=True)
+        return moc, uic, rcc
+
+    def _pkgconfig_detect(self, mods, env, kwargs):
+        modules = OrderedDict()
         for module in mods:
-            modules.append(PkgConfigDependency('Qt5' + module, environment, kwargs))
-        for m in modules:
+            modules[module] = PkgConfigDependency(self.qtpkgname + module, env, kwargs)
+        self.is_found = True
+        for m in modules.values():
+            if not m.found():
+                self.is_found = False
+                return
             self.cargs += m.get_compile_args()
             self.largs += m.get_link_args()
-        self.is_found = True
-        self.version = modules[0].modversion
+        self.version = m.modversion
+        # Try to detect moc, uic, rcc
+        if 'Core' in modules:
+            core = modules['Core']
+        else:
+            corekwargs = {'required': 'false', 'silent': 'true'}
+            core = PkgConfigDependency(self.qtpkgname + 'Core', env, corekwargs)
+        # Used by self.compilers_detect()
+        self.bindir = core.get_pkgconfig_variable('host_bins')
+        if not self.bindir:
+            # If exec_prefix is not defined, the pkg-config file is broken
+            prefix = core.get_pkgconfig_variable('exec_prefix')
+            if prefix:
+                self.bindir = os.path.join(prefix, 'bin')
 
-    def qmake_detect(self, mods, kwargs):
-        pc = subprocess.Popen(['qmake', '-v'], stdout=subprocess.PIPE,
-                              stderr=subprocess.PIPE)
-        (stdo, _) = pc.communicate()
-        if pc.returncode != 0:
+    def _find_qmake(self, qmake, env):
+        # Even when cross-compiling, if we don't get a cross-info qmake, we
+        # fallback to using the qmake in PATH because that's what we used to do
+        if env.is_cross_build():
+            qmake = env.cross_info.config['binaries'].get('qmake', qmake)
+        return ExternalProgram(qmake, silent=True)
+
+    def _qmake_detect(self, mods, env, kwargs):
+        for qmake in ('qmake-' + self.name, 'qmake'):
+            self.qmake = self._find_qmake(qmake, env)
+            if not self.qmake.found():
+                continue
+            # Check that the qmake is for qt5
+            pc = subprocess.Popen(self.qmake.fullpath +  ['-v'],
+                                  stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
+                                  universal_newlines=True)
+            stdo = pc.communicate()[0]
+            if pc.returncode != 0:
+                continue
+            if not 'Qt version ' + self.qtver in stdo:
+                mlog.log('QMake is not for ' + self.qtname)
+                continue
+            # Found qmake for Qt5!
+            break
+        else:
+            # Didn't find qmake :(
             return
-        stdo = stdo.decode()
-        if not 'version 5' in stdo:
-            mlog.log('QMake is not for Qt5.')
-            return
-        self.version = re.search('5(\.\d+)+', stdo).group(0)
-        (stdo, _) = subprocess.Popen(['qmake', '-query'], stdout=subprocess.PIPE).communicate()
+        self.version = re.search(self.qtver + '(\.\d+)+', stdo).group(0)
+        # Query library path, header path, and binary path
+        stdo = subprocess.Popen(self.qmake.fullpath +  ['-query'],
+                                stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
+                                universal_newlines=True).communicate()[0]
         qvars = {}
-        for line in stdo.decode().split('\n'):
+        for line in stdo.split('\n'):
             line = line.strip()
             if line == '':
                 continue
             (k, v) = tuple(line.split(':', 1))
             qvars[k] = v
         if mesonlib.is_osx():
-            return self.framework_detect(qvars, mods, kwargs)
+            return self._framework_detect(qvars, mods, kwargs)
         incdir = qvars['QT_INSTALL_HEADERS']
         self.cargs.append('-I' + incdir)
         libdir = qvars['QT_INSTALL_LIBS']
-        bindir = qvars['QT_INSTALL_BINS']
-        #self.largs.append('-L' + libdir)
+        # Used by self.compilers_detect()
+        self.bindir = qvars['QT_INSTALL_BINS']
+        self.is_found = True
         for module in mods:
             mincdir = os.path.join(incdir, 'Qt' + module)
             self.cargs.append('-I' + mincdir)
-            libfile = os.path.join(libdir, 'Qt5' + module + '.lib')
-            if not os.path.isfile(libfile):
-                # MinGW links directly to .dll, not to .lib.
-                libfile = os.path.join(bindir, 'Qt5' + module + '.dll')
+            if for_windows(env.is_cross_build(), env):
+                libfile = os.path.join(libdir, self.qtpkgname + module + '.lib')
+                if not os.path.isfile(libfile):
+                    # MinGW can link directly to .dll
+                    libfile = os.path.join(self.bindir, self.qtpkgname + module + '.dll')
+                    if not os.path.isfile(libfile):
+                        self.is_found = False
+                        break
+            else:
+                libfile = os.path.join(libdir, 'lib{}{}.so'.format(self.qtpkgname, module))
+                if not os.path.isfile(libfile):
+                    self.is_found = False
+                    break
             self.largs.append(libfile)
-        self.is_found = True
+        return qmake
 
-    def framework_detect(self, qvars, modules, kwargs):
+    def _framework_detect(self, qvars, modules, kwargs):
         libdir = qvars['QT_INSTALL_LIBS']
         for m in modules:
             fname = 'Qt' + m
@@ -878,7 +992,8 @@ class Qt5Dependency(Dependency):
                 self.is_found = True
                 self.cargs += fwdep.get_compile_args()
                 self.largs += fwdep.get_link_args()
-
+        # Used by self.compilers_detect()
+        self.bindir = qvars['QT_INSTALL_BINS']
 
     def get_version(self):
         return self.version
@@ -905,47 +1020,17 @@ class Qt5Dependency(Dependency):
         # Fix this to be more portable, especially to MSVC.
         return ['-fPIC']
 
-class Qt4Dependency(Dependency):
-    def __init__(self, environment, kwargs):
-        Dependency.__init__(self)
-        self.name = 'qt4'
-        self.root = '/usr'
-        self.modules = []
-        mods = kwargs.get('modules', [])
-        if isinstance(mods, str):
-            mods = [mods]
-        for module in mods:
-            self.modules.append(PkgConfigDependency('Qt' + module, environment, kwargs))
-        if len(self.modules) == 0:
-            raise DependencyException('No Qt4 modules specified.')
+class Qt5Dependency(QtBaseDependency):
+    def __init__(self, env, kwargs):
+        QtBaseDependency.__init__(self, 'qt5', env, kwargs)
 
-    def get_version(self):
-        return self.modules[0].get_version()
-
-    def get_compile_args(self):
-        args = []
-        for m in self.modules:
-            args += m.get_compile_args()
-        return args
-
-    def get_sources(self):
-        return []
-
-    def get_link_args(self):
-        args = []
-        for module in self.modules:
-            args += module.get_link_args()
-        return args
-
-    def found(self):
-        for i in self.modules:
-            if not i.found():
-                return False
-        return True
+class Qt4Dependency(QtBaseDependency):
+    def __init__(self, env, kwargs):
+        QtBaseDependency.__init__(self, 'qt4', env, kwargs)
 
 class GnuStepDependency(Dependency):
     def __init__(self, environment, kwargs):
-        Dependency.__init__(self)
+        Dependency.__init__(self, 'gnustep')
         self.modules = kwargs.get('modules', [])
         self.detect()
 
@@ -1017,7 +1102,7 @@ why. As a hack filter out everything that is not a flag."""
 
 class AppleFrameworks(Dependency):
     def __init__(self, environment, kwargs):
-        Dependency.__init__(self)
+        Dependency.__init__(self, 'appleframeworks')
         modules = kwargs.get('modules', [])
         if isinstance(modules, str):
             modules = [modules]
@@ -1037,13 +1122,14 @@ class AppleFrameworks(Dependency):
 
 class GLDependency(Dependency):
     def __init__(self, environment, kwargs):
-        Dependency.__init__(self)
+        Dependency.__init__(self, 'gl')
         self.is_found = False
         self.cargs = []
         self.linkargs = []
         try:
             pcdep = PkgConfigDependency('gl', environment, kwargs)
             if pcdep.found():
+                self.type_name = 'pkgconfig'
                 self.is_found = True
                 self.cargs = pcdep.get_compile_args()
                 self.linkargs = pcdep.get_link_args()
@@ -1066,13 +1152,14 @@ class GLDependency(Dependency):
 # sdl2-config, pkg-config and OSX framework
 class SDL2Dependency(Dependency):
     def __init__(self, environment, kwargs):
-        Dependency.__init__(self)
+        Dependency.__init__(self, 'sdl2')
         self.is_found = False
         self.cargs = []
         self.linkargs = []
         try:
             pcdep = PkgConfigDependency('sdl2', environment, kwargs)
             if pcdep.found():
+                self.type_name = 'pkgconfig'
                 self.is_found = True
                 self.cargs = pcdep.get_compile_args()
                 self.linkargs = pcdep.get_link_args()
@@ -1122,7 +1209,7 @@ class SDL2Dependency(Dependency):
 
 class ExtraFrameworkDependency(Dependency):
     def __init__(self, name, required, path=None):
-        Dependency.__init__(self)
+        Dependency.__init__(self, 'extraframeworks')
         self.name = None
         self.detect(name, path)
         if self.found():
@@ -1163,7 +1250,7 @@ class ExtraFrameworkDependency(Dependency):
 
 class ThreadDependency(Dependency):
     def __init__(self, environment, kwargs):
-        super().__init__()
+        super().__init__('threads')
         self.name = 'threads'
         self.is_found = True
         mlog.log('Dependency', mlog.bold(self.name), 'found:', mlog.green('YES'))
@@ -1173,7 +1260,7 @@ class ThreadDependency(Dependency):
 
 class Python3Dependency(Dependency):
     def __init__(self, environment, kwargs):
-        super().__init__()
+        super().__init__('python3')
         self.name = 'python3'
         self.is_found = False
         self.version = "3.something_maybe"
