@@ -70,6 +70,22 @@ class File:
     def __hash__(self):
         return hash((self.fname, self.subdir, self.is_built))
 
+def get_compiler_for_source(compilers, src):
+    for comp in compilers:
+        if comp.can_compile(src):
+            return comp
+    raise RuntimeError('No specified compiler can handle file {!s}'.format(src))
+
+def classify_unity_sources(compilers, sources):
+    compsrclist = {}
+    for src in sources:
+        comp = get_compiler_for_source(compilers, src)
+        if comp not in compsrclist:
+            compsrclist[comp] = [src]
+        else:
+            compsrclist[comp].append(src)
+    return compsrclist
+
 def flatten(item):
     if not isinstance(item, list):
         return item
@@ -165,16 +181,18 @@ def version_compare(vstr1, vstr2):
     return cmpop(varr1, varr2)
 
 def default_libdir():
-    try:
-        pc = subprocess.Popen(['dpkg-architecture', '-qDEB_HOST_MULTIARCH'],
-                              stdout=subprocess.PIPE, stderr=subprocess.DEVNULL)
-        (stdo, _) = pc.communicate()
-        if pc.returncode == 0:
-            archpath = stdo.decode().strip()
-            return 'lib/' + archpath
-    except Exception:
-        pass
-    if os.path.isdir('/usr/lib64'):
+    if is_debianlike():
+        try:
+            pc = subprocess.Popen(['dpkg-architecture', '-qDEB_HOST_MULTIARCH'],
+                                  stdout=subprocess.PIPE,
+                                  stderr=subprocess.DEVNULL)
+            (stdo, _) = pc.communicate()
+            if pc.returncode == 0:
+                archpath = stdo.decode().strip()
+                return 'lib/' + archpath
+        except Exception:
+            pass
+    if os.path.isdir('/usr/lib64') and not os.path.islink('/usr/lib64'):
         return 'lib64'
     return 'lib'
 
@@ -255,10 +273,10 @@ def do_mesondefine(line, confdata):
 
 def do_conf_file(src, dst, confdata):
     try:
-        with open(src) as f:
+        with open(src, encoding='utf-8') as f:
             data = f.readlines()
-    except Exception:
-        raise MesonException('Could not read input file %s.' % src)
+    except Exception as e:
+        raise MesonException('Could not read input file %s: %s' % (src, str(e)))
     # Only allow (a-z, A-Z, 0-9, _, -) as valid characters for a define
     # Also allow escaping '@' with '\@'
     regex = re.compile(r'[^\\]?@([-a-zA-Z0-9_]+)@')
@@ -302,14 +320,17 @@ def dump_conf_header(ofilename, cdata):
 def replace_if_different(dst, dst_tmp):
     # If contents are identical, don't touch the file to prevent
     # unnecessary rebuilds.
+    different = True
     try:
         with open(dst, 'r') as f1, open(dst_tmp, 'r') as f2:
             if f1.read() == f2.read():
-                os.unlink(dst_tmp)
-                return
+                different = False
     except FileNotFoundError:
         pass
-    os.replace(dst_tmp, dst)
+    if different:
+        os.replace(dst_tmp, dst)
+    else:
+        os.unlink(dst_tmp)
 
 def stringlistify(item):
     if isinstance(item, str):
