@@ -1,5 +1,3 @@
-#!/usr/bin/env python3
-
 # Copyright 2012-2016 The Meson development team
 
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -56,6 +54,8 @@ add_builtin_argument('werror', action='store_true')
 add_builtin_argument('layout')
 add_builtin_argument('default-library')
 add_builtin_argument('warnlevel', dest='warning_level')
+add_builtin_argument('stdsplit', action='store_false')
+add_builtin_argument('errorlogs', action='store_false')
 
 parser.add_argument('--cross-file', default=None,
                     help='File describing cross compilation environment.')
@@ -67,10 +67,10 @@ parser.add_argument('directories', nargs='*')
 
 class MesonApp():
 
-    def __init__(self, dir1, dir2, script_file, handshake, options, original_cmd_line_args):
+    def __init__(self, dir1, dir2, script_launcher, handshake, options, original_cmd_line_args):
         (self.source_dir, self.build_dir) = self.validate_dirs(dir1, dir2, handshake)
         if not os.path.isabs(options.prefix):
-            raise RuntimeError('--prefix value \'{0}\' must be an absolute path: '.format(options.prefix))
+            raise RuntimeError('--prefix value must be an absolute path: {!r}'.format(options.prefix))
         if options.prefix.endswith('/') or options.prefix.endswith('\\'):
             # On Windows we need to preserve the trailing slash if the
             # string is of type 'C:\' because 'C:' is not an absolute path.
@@ -78,7 +78,7 @@ class MesonApp():
                 pass
             else:
                 options.prefix = options.prefix[:-1]
-        self.meson_script_file = script_file
+        self.meson_script_launcher = script_launcher
         self.options = options
         self.original_cmd_line_args = original_cmd_line_args
 
@@ -89,6 +89,10 @@ class MesonApp():
     def validate_core_dirs(self, dir1, dir2):
         ndir1 = os.path.abspath(dir1)
         ndir2 = os.path.abspath(dir2)
+        if not os.path.exists(ndir1):
+            os.makedirs(ndir1)
+        if not os.path.exists(ndir2):
+            os.makedirs(ndir2)
         if not stat.S_ISDIR(os.stat(ndir1).st_mode):
             raise RuntimeError('%s is not a directory' % dir1)
         if not stat.S_ISDIR(os.stat(ndir2).st_mode):
@@ -121,12 +125,12 @@ itself as required.'''
     def check_pkgconfig_envvar(self, env):
         curvar = os.environ.get('PKG_CONFIG_PATH', '')
         if curvar != env.coredata.pkgconf_envvar:
-            mlog.log(mlog.red("WARNING:"), 'PKG_CONFIG_PATH has changed between invocations from "%s" to "%s".' %
-                     (env.coredata.pkgconf_envvar, curvar))
+            mlog.warning('PKG_CONFIG_PATH has changed between invocations from "%s" to "%s".' %
+                         (env.coredata.pkgconf_envvar, curvar))
             env.coredata.pkgconf_envvar = curvar
 
     def generate(self):
-        env = environment.Environment(self.source_dir, self.build_dir, self.meson_script_file, self.options, self.original_cmd_line_args)
+        env = environment.Environment(self.source_dir, self.build_dir, self.meson_script_launcher, self.options, self.original_cmd_line_args)
         mlog.initialize(env.get_log_dir())
         mlog.debug('Build started at', datetime.datetime.now().isoformat())
         mlog.debug('Python binary:', sys.executable)
@@ -227,6 +231,9 @@ def run_script_command(args):
     elif cmdname == 'gettext':
         import mesonbuild.scripts.gettext as abc
         cmdfunc = abc.run
+    elif cmdname == 'yelphelper':
+        import mesonbuild.scripts.yelphelper as abc
+        cmdfunc = abc.run
     else:
         raise MesonException('Unknown internal command {}.'.format(cmdname))
     return cmdfunc(cmdargs)
@@ -264,12 +271,6 @@ def run(mainfile, args):
             dir2 = args[1]
         else:
             dir2 = '.'
-    while os.path.islink(mainfile):
-        resolved = os.readlink(mainfile)
-        if resolved[0] != '/':
-            mainfile = os.path.join(os.path.dirname(mainfile), resolved)
-        else:
-            mainfile = resolved
     try:
         app = MesonApp(dir1, dir2, mainfile, handshake, options, sys.argv)
     except Exception as e:
