@@ -63,8 +63,8 @@ class RPMModule:
                 so_installed = True
             elif isinstance(target, build.StaticLibrary) and target.need_install:
                 to_delete.add('%%{buildroot}%%{_libdir}/%s' % target.get_filename())
-                mlog.log('Warning, removing', mlog.bold(target.get_filename()),
-                         'from package because packaging static libs not recommended')
+                mlog.warning('removing', mlog.bold(target.get_filename()),
+                             'from package because packaging static libs not recommended')
             elif isinstance(target, gnome.GirTarget) and target.should_install():
                 files_devel.add('%%{_datadir}/gir-1.0/%s' % target.get_filename()[0])
             elif isinstance(target, gnome.TypelibTarget) and target.should_install():
@@ -94,17 +94,16 @@ class RPMModule:
             for compiler in compiler_deps:
                 fn.write('BuildRequires: %s\n' % compiler)
             for dep in state.environment.coredata.deps:
-                fn.write('BuildRequires: pkgconfig(%s)\n' % dep)
+                fn.write('BuildRequires: pkgconfig(%s)\n' % dep[0])
             for lib in state.environment.coredata.ext_libs.values():
                 fn.write('BuildRequires: %s # FIXME\n' % lib.fullpath)
-                mlog.log('Warning, replace', mlog.bold(lib.fullpath),
-                         'with real package.',
-                         'You can use following command to find package which '
-                         'contains this lib:',
-                         mlog.bold('dnf provides %s' % lib.fullpath))
+                mlog.warning('replace', mlog.bold(lib.fullpath), 'with real package.',
+                             'You can use following command to find package which '
+                             'contains this lib:',
+                             mlog.bold('dnf provides %s' % lib.fullpath))
             for prog in state.environment.coredata.ext_progs.values():
                 if not prog.found():
-                    fn.write('BuildRequires: /usr/bin/%s # FIXME\n' %
+                    fn.write('BuildRequires: %{_bindir}/%s # FIXME\n' %
                              prog.get_name())
                 else:
                     fn.write('BuildRequires: %s\n' % ' '.join(prog.fullpath))
@@ -115,32 +114,25 @@ class RPMModule:
             if devel_subpkg:
                 fn.write('%package devel\n')
                 fn.write('Summary: Development files for %{name}\n')
-                fn.write('Requires: %{name}%{?_isa} = %{version}-%{release}\n')
+                fn.write('Requires: %{name}%{?_isa} = %{?epoch:%{epoch}:}{version}-%{release}\n')
                 fn.write('\n')
                 fn.write('%description devel\n')
                 fn.write('Development files for %{name}.\n')
                 fn.write('\n')
             fn.write('%prep\n')
             fn.write('%autosetup\n')
-            fn.write('rm -rf rpmbuilddir && mkdir rpmbuilddir\n')
             fn.write('\n')
             fn.write('%build\n')
-            fn.write('pushd rpmbuilddir\n')
-            fn.write('  %meson ..\n')
-            fn.write('  ninja-build -v\n')
-            fn.write('popd\n')
+            fn.write('%meson\n')
+            fn.write('%meson_build\n')
             fn.write('\n')
             fn.write('%install\n')
-            fn.write('pushd rpmbuilddir\n')
-            fn.write('  DESTDIR=%{buildroot} ninja-build -v install\n')
-            fn.write('popd\n')
+            fn.write('%meson_install\n')
             if len(to_delete) > 0:
-                fn.write('rm -rf %s\n' % ' '.join(to_delete))
+                fn.write('rm -vf %s\n' % ' '.join(to_delete))
             fn.write('\n')
             fn.write('%check\n')
-            fn.write('pushd rpmbuilddir\n')
-            fn.write('  ninja-build -v test\n')
-            fn.write('popd\n')
+            fn.write('%meson_test\n')
             fn.write('\n')
             fn.write('%files\n')
             for f in files:
@@ -153,7 +145,6 @@ class RPMModule:
                 fn.write('\n')
             if so_installed:
                 fn.write('%post -p /sbin/ldconfig\n')
-                fn.write('\n')
                 fn.write('%postun -p /sbin/ldconfig\n')
             fn.write('\n')
             fn.write('%changelog\n')
