@@ -37,9 +37,16 @@ def do_copy(from_file, to_file):
     shutil.copystat(from_file, to_file)
     append_to_log(to_file)
 
+def get_destdir_path(d, path):
+    if os.path.isabs(path):
+        output = destdir_join(d.destdir, path)
+    else:
+        output = os.path.join(d.fullprefix, path)
+    return output
+
 def do_install(datafilename):
-    ifile = open(datafilename, 'rb')
-    d = pickle.load(ifile)
+    with open(datafilename, 'rb') as ifile:
+        d = pickle.load(ifile)
     d.destdir = os.environ.get('DESTDIR', '')
     d.fullprefix = destdir_join(d.destdir, d.prefix)
 
@@ -56,10 +63,7 @@ def install_subdirs(data):
             src_dir = src_dir[:-1]
         src_prefix = os.path.join(src_dir, inst_dir)
         print('Installing subdir %s to %s.' % (src_prefix, dst_dir))
-        if os.path.isabs(dst_dir):
-            dst_dir = destdir_join(data.destdir, dst_dir)
-        else:
-            dst_dir = data.fullprefix + dst_dir
+        dst_dir = get_destdir_path(data, dst_dir)
         if not os.path.exists(dst_dir):
             os.makedirs(dst_dir)
         for root, dirs, files in os.walk(src_prefix):
@@ -92,27 +96,23 @@ def install_subdirs(data):
 def install_data(d):
     for i in d.data:
         fullfilename = i[0]
-        outfilename = i[1]
-        if os.path.isabs(outfilename):
-            outdir = destdir_join(d.destdir, os.path.split(outfilename)[0])
-            outfilename = destdir_join(d.destdir, outfilename)
-        else:
-            outdir = os.path.join(d.fullprefix, os.path.split(outfilename)[0])
-            outfilename = os.path.join(outdir, os.path.split(outfilename)[1])
+        outfilename = get_destdir_path(d, i[1])
+        outdir = os.path.split(outfilename)[0]
         os.makedirs(outdir, exist_ok=True)
         print('Installing %s to %s.' % (fullfilename, outdir))
         do_copy(fullfilename, outfilename)
 
 def install_man(d):
     for m in d.man:
-        outfileroot = m[1]
-        outfilename = os.path.join(d.fullprefix, outfileroot)
         full_source_filename = m[0]
+        outfilename = get_destdir_path(d, m[1])
         outdir = os.path.split(outfilename)[0]
         os.makedirs(outdir, exist_ok=True)
         print('Installing %s to %s.' % (full_source_filename, outdir))
         if outfilename.endswith('.gz') and not full_source_filename.endswith('.gz'):
-            open(outfilename, 'wb').write(gzip.compress(open(full_source_filename, 'rb').read()))
+            with open(outfilename, 'wb') as of:
+                with open(full_source_filename, 'rb') as sf:
+                    of.write(gzip.compress(sf.read()))
             shutil.copystat(full_source_filename, outfilename)
             append_to_log(outfilename)
         else:
@@ -121,8 +121,8 @@ def install_man(d):
 def install_headers(d):
     for t in d.headers:
         fullfilename = t[0]
-        outdir = os.path.join(d.fullprefix, t[1])
         fname = os.path.split(fullfilename)[1]
+        outdir = get_destdir_path(d, t[1])
         outfilename = os.path.join(outdir, fname)
         print('Installing %s to %s' % (fname, outdir))
         os.makedirs(outdir, exist_ok=True)
@@ -142,7 +142,8 @@ def run_install_script(d):
         print('Running custom install script %s' % script)
         suffix = os.path.splitext(script)[1].lower()
         if platform.system().lower() == 'windows' and suffix != '.bat':
-            first_line = open(script, encoding='latin_1', errors='ignore').readline().strip()
+            with open(script, encoding='latin_1', errors='ignore') as f:
+                first_line = f.readline().strip()
             if first_line.startswith('#!'):
                 if shutil.which(first_line[2:]):
                     commands = [first_line[2:]]
@@ -194,9 +195,9 @@ def check_for_stampfile(fname):
 def install_targets(d):
     for t in d.targets:
         fname = check_for_stampfile(t[0])
-        outdir = os.path.join(d.fullprefix, t[1])
-        aliases = t[2]
+        outdir = get_destdir_path(d, t[1])
         outname = os.path.join(outdir, os.path.split(fname)[-1])
+        aliases = t[2]
         should_strip = t[3]
         install_rpath = t[4]
         print('Installing %s to %s' % (fname, outname))
